package net.fukure.flazrdumper;

import java.awt.EventQueue;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Properties;
import java.util.TreeMap;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JTextArea;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.Element;

import net.fukure.flazrdumper.io.ChunkedInputStream;
import net.fukure.flazrdumper.io.Util;


public class Server extends Thread {

	private ServerSocket httpServer = null;
	private Socket clientSocket = null;
	private int defaultPort = 8888;
	private int port = defaultPort;
	private GuiWB gui;
	private Flazr flazr;
	private RtmpDump rtmpdump;
	private String[] args;
	private Properties prop;
	public String livestream_url = "rtmp://localhost/live/livestream";
	public String debug_flazr_path = "D:\\FlazrDumper\\flazr-0.7-RC2";
	public String debug_rtmpdump_path = "D:\\FlazrDumper\\rtmpdump-2.3";

	private ArrayList<byte[]> headerList = new ArrayList<byte[]>();
	private TreeMap<Integer, byte[]> dataList = new TreeMap<Integer, byte[]>();
	private ArrayList<String> rtmpdumpLogList = new ArrayList<String>();
	
	public static void main(String[] args) {
		Server server = new Server();
		server.args = args;
		server.start();
	}
	
	private void setLookAndFeel(){
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			//e1.printStackTrace();
		}
	}
	
	void init(){
		
		setLookAndFeel();
		
		loadState();
		
		setOnExitListener();
		
		try{
			port = Integer.parseInt(prop.getProperty("port"));
		}catch(Exception e){
			port = defaultPort;
		}
		try{
			if(args.length>0){
				port = Integer.parseInt(args[0]);
				if(port<1024) port = defaultPort;
				if(port>65535) port = defaultPort;
			}
		}catch(Exception e){
			port = defaultPort;
		}
		
		livestream_url = (prop==null) ? livestream_url : prop.getProperty("rtmpDumpUrl", livestream_url);
		
		try{
			gui = new GuiWB();
			gui.setTitle("FlazrDumper");
			//gui.setSize(320, 350);
			gui.rtmpDumpUrlTextField.setText(livestream_url);
			gui.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			gui.setVisible(true);
		}catch(Exception e){
			gui = null;
		}
		
	}
	
	private void loadState(){
		
		SaveProperties sp = new SaveProperties();
		prop = sp.load();
	}
	
	private void saveState(){

		if(gui!=null){
			livestream_url = gui.rtmpDumpUrlTextField.getText();
		}
		
		Properties prop = new Properties();
		prop.setProperty("rtmpDumpUrl", livestream_url);
		prop.setProperty("port", String.valueOf(port));

		SaveProperties sp = new SaveProperties();
		sp.save(prop);
			
	}
	
	private void setOnExitListener(){
		Runtime.getRuntime().addShutdownHook(new Thread() {
			public void run() {
				
				saveState();
				
				final Thread main = Thread.currentThread();
				System.out.println("Starting shutdown...");
				System.out.flush();
				if(flazr!=null){
					flazr.kill();
				}
				if(rtmpdump!=null){
					rtmpdump.kill();
				}
				
				main.interrupt();
				try {
					main.join(3000);
				} catch (InterruptedException e) {
				}
				System.out.println("Done.");
				System.out.flush();
			}
		});
	}
	
	@Override
	public void run() {
		
		//check();
		
		init();

		GuiThread guiThread = new GuiThread();
		guiThread.start();
		
		flazr = new Flazr();
		flazr.start();
		
		try {
            System.out.println("http:"+port);
            httpServer = new ServerSocket();
            try{
            	httpServer.bind(new InetSocketAddress(port));
            }catch (IOException e) {
                bindError();
            }
            
            while(true){
            	clientSocket = httpServer.accept();
            	new Client(this, clientSocket);
            }
        }catch (Exception e) {
            System.out.println(e);
        }
		
	}
	
	class GuiThread extends Thread{
		private JTextArea flazrLogTextArea;

		@Override
		public void run() {
			try {
				flazrLogTextArea = gui.flazrTextArea;
				flazrLogTextArea.setEditable(false);
				flazrLogTextArea.getDocument().addDocumentListener(new DocumentListener() {
				  @Override public void insertUpdate(DocumentEvent e) {
				    final Document doc = flazrLogTextArea.getDocument();
				    final Element root = doc.getDefaultRootElement();
				    if(root.getElementCount()>100){
					  EventQueue.invokeLater(new Runnable() {
					    @Override public void run() {
					      removeLines(doc, root);
					    }
					  });
				    }
				    Element fl = root.getElement(root.getElementCount()-1);
				    //flazrLogTextArea.setCaretPosition(doc.getLength());
				    flazrLogTextArea.setCaretPosition(fl.getStartOffset());
				  }
				  private void removeLines(Document doc, Element root) {
				    Element fl = root.getElement(0);
				    try{
				      doc.remove(0, fl.getEndOffset());
				    }catch(BadLocationException ble) {
				      System.out.println(ble);
				    }
				  }
				  @Override public void removeUpdate(DocumentEvent e) {}
				  @Override public void changedUpdate(DocumentEvent e) {}
				});
				
				while(true){
					StringBuffer sb = new StringBuffer();
					for (int i = 0; i < rtmpdumpLogList.size(); i++) {
						sb.append(rtmpdumpLogList.get(i)).append("\n");
					}
					if(gui!=null){
						gui.rtmpDumpTextArea.setText(sb.toString().trim());
					}
										
					Thread.sleep(500);
				}
			} catch (InterruptedException e) {
				//e.printStackTrace();
			}
		}
	}
	
    class Client extends Thread{

		private Server server;
    	private Socket socket = null;
        private BufferedInputStream is;
        private BufferedOutputStream os;
        private boolean dumper = false;
        private long start = 0;
        
    	public Client(Server server, Socket clientSocket) {
    		this.server = server;
    		this.socket = clientSocket;
    		start();
		}
    	
    	@Override
		public void run() {
			try {
				
				//setPriority(Thread.MIN_PRIORITY);
				
				System.out.println("http open:"+socket);
				
				is = new BufferedInputStream(socket.getInputStream());
		        os = new BufferedOutputStream(socket.getOutputStream());

	        	Http http = new Http(is, os);
		        http.readHeader(is);
		        
		        System.out.println(http.header);
		        
		        if(http.header.startsWith("GET")){
		        	//peercaststation, player
		        	
		        	if(!http.path.equals("/") && !http.path.equals("/livestream.flv") && !http.path.equals("/live/livestream.flv")){
		        		http.fileResponse();
		        		throw new Exception();
		        	}
		        	
		        	if(rtmpdump==null){
		        		dumper = true;
		        	}
		        	
	        		startRtmpDump();
	        		
	        		for (int i = 0; i < 50; i++) {
	        			if(!rtmpdump.isRunning() || dataList.size()>0){
	        				break;
	        			}
						Thread.sleep(100);
					}
		        	
		        	if(!isReady()){
						os.write("HTTP/1.1 404 Not Found\r\n".getBytes());
						os.write("Content-Type: video/x-flv\r\n".getBytes());
						os.write("Content-Length: 0\r\n".getBytes());
						os.write("Connection: close\r\n\r\n".getBytes());
						os.flush();
		        		throw new IOException();
		        	}else{
		        		
						os.write("HTTP/1.1 200 OK\r\n".getBytes());
						os.write("Content-Type: video/x-flv\r\n".getBytes());
						os.write("Connection: close\r\n\r\n".getBytes());
						os.flush();
						
		                sendDataLoop();
		        	}
		        }
		        
			} catch (Exception e) {
				//e.printStackTrace();
			}
		
			try {
				System.out.println("http close:"+socket);
				is.close();
		        os.close();
			} catch (Exception e) {
				e.printStackTrace();
			}
			
			stopRtmpDump();
			
		}
    	
    	void stopRtmpDump(){
			if(dumper && rtmpdump!=null){
				rtmpdump.kill();
				rtmpdump = null;
			}
    	}
    	
    	RtmpDump startRtmpDump(){
    		if(rtmpdump==null){
	    		rtmpdump = new RtmpDump();
	    		rtmpdump.start();
    		}
    		return rtmpdump;
    	}
    	
    	byte[] timeshift(byte[] data){
    		long t = data[7] << 24 | data[4] << 16 | data[5] << 8 | data[6];
    		long time = t;
    		if(start>0){
    			time = t - start;
    		}else{
    			start = time;
    		}
    		data[7] = (byte) ((time >> 24) & 0xFF);
    		data[4] = (byte) ((time >> 16) & 0xFF);
    		data[5] = (byte) ((time >> 8) & 0xFF);
    		data[6] = (byte) ((time) & 0xFF);
    		
    		return data;
    	}
		
		void sendDataLoop() throws IOException, InterruptedException {
			
			for (int i = 0; i < headerList.size(); i++) {
				os.write(headerList.get(i));
			}
			os.flush();
        	
            int latest = server.dataList.lastKey();
            int wait = 0;
            int nowait = 0;
            while(true){
            	if(rtmpdump==null) break;
            	if(server.dataList.size()==0) break;
            	if(latest<server.dataList.firstKey()) break;
            	byte[] data = server.dataList.get(latest);
            	if(data!=null){
                	os.write(data); //timeshift(data)
                	//os.flush();
                	wait = 0;
    				nowait++;
    				if(nowait>50){
    					Thread.sleep(10);
    					nowait = 0;
    				}
                	latest++;
            	}else{
					Thread.sleep(50);
					wait++;
					nowait = 0;
                	if(wait>200){
                		//System.out.println("timeout");
                		break;
                	}
            	}
            }
            
		}

		boolean isReady(){
			if(headerList.size()==0 || dataList.size()==0){
				return false;
			}else{
				return true;
			}
		}
    }
	
	class RtmpDump extends Thread {
		private InputStream is = null;
		private Process process = null;
		private boolean playing = false;
		private boolean running = true;
		private ByteArrayOutputStream out = new ByteArrayOutputStream();
		private ByteBuffer bb = ByteBuffer.allocate(8*1024*1024);
		@Override
		public void run() {
			try {
				List<String> list = getOption(getLiveStreamUrl());
				ProcessBuilder pb = new ProcessBuilder(list);
				//pb.redirectErrorStream(true);
				
				/*
				String path = get_currentpath() + System.getProperty("file.separator");
				
				File dir = new File(path + "rtmpdump-2.3");
				if(!dir.isDirectory()){
					dir = new File(debug_rtmpdump_path+"\\rtmpdump.exe");
				}
				//pb.directory(dir);
				 */
				
				process = pb.start();
				
				InputStream is = process.getInputStream();
				InputStream es = process.getErrorStream();
				new ErrorStream(es).start();
				new RecordStream().start();
				byte[] buff = new byte[8192];
				int r;
				while ((r = is.read(buff)) > -1) {
					//String s = new String(buff, 0, r);
					if(buff[0]==0x46 && buff[1]==0x4C && buff[2]==0x56){
						playing = true;
					}
					if(playing){
						out.write(buff, 0, r);
						write(out.toByteArray());
						out.reset();
						//System.out.println(s.length());
						//System.out.println(Util.hexString(buff, 16));
					}
				}
			} catch (IOException e) {
				//e.printStackTrace();
			}
			try {
				if(is!=null){
					is.close();
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
			
			running = false;
			
		}
		
		boolean isRunning(){
			return running;
		}
		
		class RecordStream extends Thread {
			
			long time = 0;
			ByteArrayOutputStream out = new ByteArrayOutputStream();
			int readHead() throws IOException{
				byte[] b = read(11);
				byte[] d = new byte[3];
				System.arraycopy(b, 1, d, 0, 3);
				out.write(b);
				return Util.intFrom3bytes(d);
			}
			
			byte[] readBody(int size) throws IOException{
				byte[] b = read(size);
				out.write(b);
				return b;
			}
			
			int readFoot() throws IOException{
				byte[] b = read(4);
				out.write(b);
				return Util.intFrom4bytes(b);
			}
			
			@Override
			public void run() {

				try {
					
					headerList.clear();
					dataList.clear();
					
					boolean hasMetaData = false;
					int seq = 0;
					
					byte[] data = read(13);
					//System.out.println(Util.hexString(data, 16));
					byte[] body;
					headerList.add(data);
					
					while(true){
						int size = readHead();
						body = readBody(size);
						int prevsize = readFoot();
						if(size+11!=prevsize){
							throw new IOException();
						}
						data = out.toByteArray();
						
						secIntervalOut(data);

						/*
						if(seq<10){
							System.out.println(Util.hexString(data, 16));
							System.out.println(Util.hexString(body, 16));
						}
						*/
						
						if(seq<10 && isMetaData(body)||isAVCHeader(body)||isAACHeader(body)){
							if(isMetaData(body)){
								hasMetaData = true;
							}
							synchronized (headerList) {
								headerList.add(data);
								out.reset();
							}
						}else{
							if(!hasMetaData) break;
							synchronized (dataList) {
								dataList.put(seq++, data);
								if(dataList.size()>200){
									dataList.remove(dataList.firstKey());
								}
								out.reset();
							}
						}
						
						//break;
					}
				} catch (IOException e) {
					//e.printStackTrace();
				}

				headerList.clear();
				dataList.clear();
				
			}
			
			void secIntervalOut(byte[] data){
				String s = Util.hexString(data, 16);
				//if(System.currentTimeMillis()-time>500){
					//System.out.println(s);
					time = System.currentTimeMillis();
				//}
				rtmpdumpLogList.add(s);
				if(rtmpdumpLogList.size()>3){
					rtmpdumpLogList.remove(0);
				}
			}

			boolean isAVCHeader(byte[] data){
				if(data[0]==0x17 && data[1]==0x00 && data[2]==0x00 && data[2] == 0x00 && data[3] == 0x00){
					return true;
				}else{
					return false;
				}
			}

			boolean isAACHeader(byte[] data){
				if((data[0] & 0xff)==0xaf && data[1]==0x00){
					return true;
				}else{
					return false;
				}
			}

			boolean isMetaData(byte[] data){
				if(data[0]==0x02 && data[1]==0x00 && data[2]==0x0A &&
					data[3]==0x6F && data[4]==0x6E &&
					data[5]==0x4D && data[6]==0x65 && data[7]==0x74 && data[8]==0x61 &&
					data[9]==0x44 && data[10]==0x61 && data[11]==0x74 && data[12]==0x61){
					return true;
					//on
					//Meta
					//Data
				}
				return false;
			}
		}
		
		class ErrorStream extends Thread {
			BufferedReader br;
			InputStream is;
			public ErrorStream(InputStream is) {
				this.is = is;
				this.br = new BufferedReader(new InputStreamReader(is));
			}
			@Override
			public void run() {
				try{
					String line = null;
					while((line=br.readLine())!=null){
						//System.out.println(line);
						Thread.sleep(20);
					}
				}catch(Exception e){
					e.printStackTrace();
				}
			}
		}
		
		public void kill() {
			
			bb = null;
			
			try {
				Thread.sleep(500);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			System.out.println("rtmpdump kill");
			
			if(process!=null){
				process.destroy();
			}
			System.out.println("rtmpdump killed");
			
			rtmpdumpLogList.clear();
		}

		private List<String> getOption(String url){
			
			String sp = System.getProperty("file.separator");

			File file = new File("rtmpdump-2.3" + sp + Server.getRtmpDumpFile());
			if(!file.isFile()){
				file = new File(debug_rtmpdump_path + sp + "rtmpdump.exe");
			}
			
			String filepath = file.getAbsolutePath();
			String rtmpdump = filepath+" -v -r "+url;
			System.out.println(rtmpdump);
			String[] options = rtmpdump.split(" ");
			return Arrays.asList(options);
		}

		public byte[] read(int len) throws IOException {
			byte[] buff = new byte[len];
			try {
				while(true){
					synchronized (bb) {
						if(bb.position()>=len){
							//System.out.println("read1 "+bb.position()+"p");
							bb.flip();
							bb.get(buff);
							//System.out.println("read"+len);
							bb.compact();
							//System.out.println("read2 "+bb.position()+"p");
							break;
						}
					}
					Thread.sleep(20);
				}
			} catch (InterruptedException e) {
				e.printStackTrace();
			} catch (Exception e) {
				throw new IOException();
			}
			return buff;
		}

		private void write(byte[] buff) throws IOException{
			try{
				synchronized (bb) {
					if(bb.position()+buff.length<=bb.capacity()){
						//System.out.println("write1 "+bb.position()+"p");
						bb.put(buff);
						//System.out.println("write"+buff.length);
						//System.out.println("write2 "+bb.position()+"p");
					}else{
						throw new IOException();
					}
				}
			}catch(Exception e){
				throw new IOException();
			}
		}
		
	}
	
	class Flazr extends Thread {
		Process p = null;
		Process kp = null;
		boolean copyright = false;
		@Override
		public void run() {
			
			p = exec(getOption(), true);

		}
		
		Process exec(List<String> list, boolean async){
			ProcessBuilder pb = new ProcessBuilder(list);
			//pb.redirectErrorStream(true);

			File dir = new File("flazr-0.7-RC2");
			if(!dir.isDirectory()){
				dir = new File(debug_flazr_path);
			}
			pb.directory(dir);

			try {
				p = pb.start();

				InputStreamThread ist = new InputStreamThread(p.getInputStream());
				InputStreamThread est = new InputStreamThread(p.getErrorStream());
				est.setErrorStream(true);
				ist.start();
				est.start();
				
				if(!async){
					ist.join();
					est.join();
				}
								
			} catch (IOException e) {
				e.printStackTrace();
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			
			return p;
		}
		
		void kill(){
			System.out.println("flazr kill");
			
			try {
				Thread t = new Thread(new Runnable() {
					@Override
					public void run() {
						try {
							kp = exec(getKillOption(), false);
							kp.waitFor();
							kp.destroy();
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
					}
				});
				t.start();
				t.join(3000);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			
			if(kp!=null){
				kp.destroy();
			}
			
			try{
				p.getInputStream().close();
				p.getErrorStream().close();
			} catch (Exception e) {
				e.printStackTrace();
			}
			
			if(p!=null){
				p.destroy();
			}
			System.out.println("flazr killed");
		}
		
		void logout6(String msg){
			System.out.println(msg);
			if(gui!=null){
				msg = msg.replaceAll(" \\[.*?\\] .... \\[.*?\\]" , "");
				gui.flazrTextArea.append((gui.flazrTextArea.getDocument().getLength()>0)?"\n"+msg:msg);
			}
		}
		
		public class InputStreamThread extends Thread {

			private boolean errorstream = false;
			private BufferedReader br;
			private int count = 0;

			public InputStreamThread(InputStream is) {
				this.br = new BufferedReader(new InputStreamReader(is));
			}

			public InputStreamThread(InputStream is, String charset) {
				try {
					this.br = new BufferedReader(new InputStreamReader(is, charset));
				} catch (UnsupportedEncodingException e) {
					throw new RuntimeException(e);
				}
			}
			
			public void setErrorStream(boolean bool){
				errorstream = bool;
			}

			@Override
			public void run() {
				try {
					for (;;) {
						String line = br.readLine();
						if (line == null) 	break;
						if(count<100){
							if(line.indexOf("java.net.BindException")>0){
								bindError();
							}
							if(line.indexOf("http://www.gnu.org/licenses/")>0){
								copyright = true;
							}
						}
						if(errorstream){
							logout6(line);
						}else{
							for (int i = 0; i < 5; i++) {
								if(copyright) break;
								try {
									Thread.sleep(100);
								} catch (InterruptedException e) {
								}
							}
							logout6(line);
						}
						
						count++;
					}
				} catch (IOException e) {
					throw new RuntimeException(e);
				}
				
				try {
					br.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
				
			}

		}
		
		private List<String> getOption(){
			String flazr = "java -Xmx128m -cp conf;lib\\commons-cli-1.2.jar;lib\\commons-codec-1.2.jar;lib\\commons-httpclient-3.1.jar;lib\\flazr-0.7-RC2.jar;lib\\jcl104-over-slf4j-1.4.2.jar;lib\\log4j-1.2.14.jar;lib\\netty-3.1.5.GA.jar;lib\\slf4j-api-1.4.2.jar;lib\\slf4j-log4j12-1.4.2.jar -Dflazr.log.suffix=-server com.flazr.rtmp.server.RtmpServer %*";
			if(!Server.isWindows()){
				flazr = "java -Xmx128m -cp conf:lib/commons-cli-1.2.jar:lib/commons-codec-1.2.jar:lib/commons-httpclient-3.1.jar:lib/flazr-0.7-RC2.jar:lib/jcl104-over-slf4j-1.4.2.jar:lib/log4j-1.2.14.jar:lib/netty-3.1.5.GA.jar:lib/slf4j-api-1.4.2.jar:lib/slf4j-log4j12-1.4.2.jar -Dflazr.log.suffix=-server com.flazr.rtmp.server.RtmpServer $*";
			}
			System.out.println(flazr);
			String[] options = flazr.split(" ");
			return Arrays.asList(options);
		}
		
		
		private List<String> getKillOption(){
			String flazr = "java -cp conf;lib\\commons-cli-1.2.jar;lib\\commons-codec-1.2.jar;lib\\commons-httpclient-3.1.jar;lib\\flazr-0.7-RC2.jar;lib\\jcl104-over-slf4j-1.4.2.jar;lib\\log4j-1.2.14.jar;lib\\netty-3.1.5.GA.jar;lib\\slf4j-api-1.4.2.jar;lib\\slf4j-log4j12-1.4.2.jar -Dflazr.log.suffix=-stop com.flazr.rtmp.server.ServerStop %*";
			if(!Server.isWindows()){
				flazr = "java -cp conf:lib/commons-cli-1.2.jar:lib/commons-codec-1.2.jar:lib/commons-httpclient-3.1.jar:lib/flazr-0.7-RC2.jar:lib/jcl104-over-slf4j-1.4.2.jar:lib/log4j-1.2.14.jar:lib/netty-3.1.5.GA.jar:lib/slf4j-api-1.4.2.jar:lib/slf4j-log4j12-1.4.2.jar -Dflazr.log.suffix=-stop com.flazr.rtmp.server.ServerStop $*";
			}
			System.out.println(flazr);
			String[] options = flazr.split(" ");
			return Arrays.asList(options);
		}

	}

	private void bindError() {

		JFrame frame = new JFrame();
		String msg = "bind error.\nport is already in use.";
		JOptionPane.showMessageDialog(frame, msg);
		
	}
	
	public String getLiveStreamUrl() {
		try{
			return gui.rtmpDumpUrlTextField.getText();
		}catch(Exception e){
			return livestream_url;
		}
	}

	void check(){

		JFrame frame = new JFrame();
		String msg = "";
		File flazr = new File("flazr-0.7-RC2/lib/flazr-0.7-RC2.jar");
		if(!flazr.isFile()){
			msg += "flazr-0.7-RC2.jar not found.\n";
		}
		File rtmpdump = new File("rtmpdump-2.3/"+getRtmpDumpFile());
		if(!rtmpdump.isFile()){
			msg += getRtmpDumpFile()+" not found.\n";
		}
		if(msg.length()>0){
			JOptionPane.showMessageDialog(frame, msg);
			System.exit(0);
		}
		
	}
	
	private static boolean isWindows(){
		String osName = System.getProperty("os.name").toLowerCase();
		if(osName.indexOf("windows")>=0){
			return true;
		}else{
			return false;
		}
	}

	String get_currentpath(){

		String cp = System.getProperty("java.class.path");
		String fs = System.getProperty("file.separator");
		String acp = (new File(cp)).getAbsolutePath();

		int p,q;
		for(p=0;(q=acp.indexOf(fs,p))>=0;p=q+1);

		return acp.substring(0,p);

	}
	
	static String getRtmpDumpFile(){
		if(Server.isWindows()){
			return "rtmpdump.exe";
		}
		if(Server.getOsBit()==64){
			return "rtmpdump_x86_64";
		}else{
			return "rtmpdump";
		}
	}
	
	static int getOsBit() {
        String os = System.getProperty( "sun.arch.data.model" ) ;
        if( os != null && ( os = os.trim() ).length() > 0 ) {
            if( "32".equals( os ) ) {
                return 32 ;
            }
            else if( "64".equals( os ) ) {
                return 64 ;
            }
        }
        os = System.getProperty( "os.arch" ) ;
        if( os == null || ( os = os.trim() ).length() <= 0 ) {
            return -1 ;
        }
        if( os.endsWith( "86" ) ) {
            return 32 ;
        }
        else if( os.endsWith( "64" ) ) {
            return 64 ;
        }
        return 32 ;
    }
	
}
